/*
 *  GeoServer-Manager - Simple Manager Library for GeoServer
 *  
 *  Copyright (C) 2010 GeoSolutions S.A.S.
 *  http://www.geo-solutions.it
 *
 *  GPLv3 + Classpath exception
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.geosolutions.geoserver.rest.decoder;

import it.geosolutions.geoserver.rest.decoder.utils.JDOMBuilder;
import java.util.List;
import org.jdom.Element;

/**
 * Parse <TT>DataStore</TT>s returned as XML REST objects.
 * <P>
 * This is the XML document returned by GeoServer when requesting a DataStore:
 * <PRE>
 * {@code
<dataStore>
    <name>sf</name>
    <enabled>true</enabled>
    <workspace>
        <name>sf</name>
        <atom:link xmlns:atom="http://www.w3.org/2005/Atom" rel="alternate"
            href="http://localhost:8080/geoserver/rest/workspaces/sf.xml"
            type="application/xml"/>
    </workspace>
    <connectionParameters>
        <entry key="namespace">http://www.openplans.org/spearfish</entry>
        <entry key="url">file:data/sf</entry>
    </connectionParameters>
    <featureTypes>
        <atom:link xmlns:atom="http://www.w3.org/2005/Atom" rel="alternate"
            href="http://localhost:8080/geoserver/rest/workspaces/sf/datastores/sf/featuretypes.xml"
            type="application/xml"/>
    </featureTypes>
</dataStore>
 * }
 * </PRE>
 * <I>Note: the whole XML fragment is stored in memory. At the moment, there are
 * methods to retrieve only the more useful data.
 * @author etj
 */
public class RESTDataStore {

    private final Element dsElem;

    public enum DBType {

        POSTGIS("postgis"),
        SHP("shp"),
        UNKNOWN(null);
        private final String restName;

        private DBType(String restName) {
            this.restName = restName;
        }

        public static DBType get(String restName) {
            for (DBType type : values()) {
                if (type == UNKNOWN) {
                    continue;
                }
                if (type.restName.equals(restName)) {
                    return type;
                }
            }
            return UNKNOWN;
        }
    };

    public static RESTDataStore build(String xml) {
        if (xml == null) {
            return null;
        }

        Element e = JDOMBuilder.buildElement(xml);
        if (e != null) {
            return new RESTDataStore(e);
        } else {
            return null;
        }
    }

    protected RESTDataStore(Element dsElem) {
        this.dsElem = dsElem;
    }

    public String getName() {
        return dsElem.getChildText("name");
    }

    public String getWorkspaceName() {
        return dsElem.getChild("workspace").getChildText("name");
    }

    protected String getConnectionParameter(String paramName) {
        Element elConnparm = dsElem.getChild("connectionParameters");
        if (elConnparm != null) {
            for (Element entry : (List<Element>) elConnparm.getChildren("entry")) {
                String key = entry.getAttributeValue("key");
                if (paramName.equals(key)) {
                    return entry.getTextTrim();
                }
            }
        }

        return null;
    }

    public DBType getType() {
        return DBType.get(getConnectionParameter("dbtype"));
    }
}
