/*
 *  GeoServer-Manager - Simple Manager Library for GeoServer
 *  
 *  Copyright (C) 2010 GeoSolutions S.A.S.
 *  http://www.geo-solutions.it
 *
 *  GPLv3 + Classpath exception
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package it.geosolutions.geoserver.rest.encoder;

import java.util.HashMap;
import java.util.Map;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Creates an XML document by mapping properties to XML nodes.<br/>
 * You can set the root element in the constructor.
 * Any key/value pair will be encoded as <key>value</key> node.
 * Any key containing one or more slash ("/") will be encoded as nested nodes;
 * <P>e.g.:</P>
 * <PRE> {@code key = "k1/k2/k3", value = "value" }</pre>
 * will be encoded as
 * <PRE> {@code <k1><k2><k3>value</k3></k2></k1> }</pre>
 * 
 * @author ETj (etj at geo-solutions.it)
 */
public class PropertyXMLEncoder {

    private final static XMLOutputter OUTPUTTER = new XMLOutputter(Format.getCompactFormat());
    private final Map<String, String> configElements = new HashMap<String, String>();
    private final String rootName;

    public PropertyXMLEncoder(String rootName) {
        this.rootName = rootName;
    }
        
    protected void setOrRemove(String key, String value) {
        if (value != null) {
            configElements.put(key, value);
        } else {
            configElements.remove(key);
        }
    }

    protected void set(String key, String value) {
        setOrRemove(key, value);
    }

    public boolean isEmpty() {
        return configElements.isEmpty();
    }

    public String encodeXml() {

        Element layer = new Element(rootName);
        for (String key : configElements.keySet()) {
            final String value = configElements.get(key);
            add(layer, key, value);
        }

        return OUTPUTTER.outputString(layer);
    }
    
    private void add(Element e, String key, String value) {
        if( ! key.contains("/") ) {
            e.addContent(new Element(key).setText(value));
        } else {
            int i = key.indexOf("/");
            String childName = key.substring(0,i);
            String newkey = key.substring(i+1);
            
            Element child = e.getChild(childName);
            if(child == null) {
                child = new Element(childName);
                e.addContent(child);
            }
            
            add(child, newkey, value);        
        }

    }
}
